from __future__ import annotations
from irctokens.line import build, Line
from ircrobots.bot import Bot as BaseBot
from ircrobots.server import Server as BaseServer
from ircrobots.params import ConnectionParams


from .interface import IServer
from .yobot import Yobot


class Server(BaseServer, IServer):
    bot: Bot

    async def line_read(self, line: Line) -> None:
        if line.command not in ("PING", "PONG", "AWAY"):
            print(f"{self.name} < {line.format()}")
        me = self.nickname_lower
        if (
            line.command == "PRIVMSG"
            and self.has_channel(line.params[0])
            and line.hostmask is not None
            and self.casefold(line.hostmask.nickname) != me
            and self.has_user(line.hostmask.nickname)
        ):
            await self.bot.yobot.handle_irc(
                self, self.name, self.casefold(line.params[0]), line
            )

    async def line_send(self, line: Line) -> None:
        if line.command not in ("PING", "PONG"):
            print(f"{self.name} > {line.format()}")

    async def reply(self, line: Line, msg: str) -> None:
        await self.send(build("PRIVMSG", [line.params[0], msg]))


class Bot(BaseBot):
    def __init__(self, yobot: Yobot) -> None:
        self.yobot = yobot
        super().__init__()

    def create_server(self, name: str) -> BaseServer:
        return Server(self, name)


async def main(yobot: Yobot) -> None:
    bot = Bot(yobot)
    for name, network in yobot.config["irc"].items():
        params = ConnectionParams(
            "yobot",
            network["host"],
            network["port"],
            autojoin=list(network["channels"].keys()),
        )
        await bot.add_server(name, params)

    await bot.run()
