import httpx


from asyncio import timeout

from yobot.interface import ILine, IServer, IPlugin


async def req(token: str, project: int, station: int) -> str:
    async with httpx.AsyncClient() as client:
        headers = {
            "Accept": "application/json",
            "Authorization": f"Bearer {token}",
        }
        r = await client.get(
            f"https://api-v2.acrcloud.com/api/bm-bd-projects/{project}/channels/{station}/realtime_results",
            headers=headers,
        )
        j = r.json()
        if "music" not in j["data"]["metadata"]:
            return r"snob2000 now playing: ¯\_(ツ)_/¯"
        m = j["data"]["metadata"]["music"][0]
        title = m["title"]
        artist = " & ".join([a["name"] for a in m["artists"]])
        return f"snob2000 np: {artist} - {title}"


class Plugin(IPlugin):
    def __init__(self, config: dict) -> None:
        super().__init__(config)

    async def handle(self, bot: IServer, line: ILine) -> None:
        msg = line.params[1]
        if msg == "!s2k":
            async with timeout(10):
                np = await req(**self.config)
                await bot.reply(line, np)
